/**
  ******************************************************************************
  * @file    x_nucleo_iks01a2.h
  * @author  MEMS Application Team
  * @brief   This file contains definitions for the x_nucleo_iks01a2.c
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2018 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __X_NUCLEO_IKS01A2_H
#define __X_NUCLEO_IKS01A2_H

#ifdef __cplusplus
extern "C" {
#endif



/* Includes ------------------------------------------------------------------*/

#ifdef USE_STM32F4XX_NUCLEO
#include "stm32f4xx_hal.h"
#endif

#ifdef TARGET_STM32L0
#include "stm32l0xx_hal.h"
#endif

#ifdef USE_STM32L1XX_NUCLEO
#include "stm32l1xx_hal.h"
#endif

#ifdef USE_STM32L4XX_NUCLEO
#include "stm32l4xx_hal.h"
#endif

#ifdef TARGET_STM32L0
#include "stm32l0xx_hal.h"
#endif

#include "accelerometer.h"
//#include "gyroscope.h"
//#include "magnetometer.h"
//#include "humidity.h"
//#include "temperature.h"
//#include "pressure.h"

/** @addtogroup BSP BSP
 * @{
 */

/** @addtogroup X_NUCLEO_IKS01A2 X_NUCLEO_IKS01A2
 * @{
 */

/** @addtogroup X_NUCLEO_IKS01A2_IO IO
 * @{
 */

/** @addtogroup X_NUCLEO_IKS01A2_IO_Public_Constants Public constants
 * @{
 */

#define IKS01A2_H3LIS331DL_AIS328DQ_AIS3624DQ_WHO_AM_I                (uint8_t)0x32
#define IKS01A2_LSM303AGR_ACC_LIS2DH12_WHO_AM_I                       (uint8_t)0x33
#define IKS01A2_LSM303AGR_MAG_LIS2MDL_IIS2MDC_ISM303DAC_MAG_WHO_AM_I  (uint8_t)0x40
#define IKS01A2_ISM303DAC_ACC_WHO_AM_I                                (uint8_t)0x43
#define IKS01A2_LIS2DW12_IIS2DLPC_WHO_AM_I                            (uint8_t)0x44
#define IKS01A2_LSM6DSL_ISM330DLC_WHO_AM_I                            (uint8_t)0x6A
#define IKS01A2_LSM6DSR_WHO_AM_I                                      (uint8_t)0x6B
#define IKS01A2_LSM6DSO_WHO_AM_I                                      (uint8_t)0x6C
#define IKS01A2_LPS22HB_LPS33HW_WHO_AM_I                              (uint8_t)0xB1
#define IKS01A2_LPS22HH_WHO_AM_I                                      (uint8_t)0xB3
#define IKS01A2_HTS221_WHO_AM_I                                       (uint8_t)0xBC
#define IKS01A2_A3G4250D_WHO_AM_I                                     (uint8_t)0xD3

/* I2C clock speed configuration (in Hz) */
#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)))
#define NUCLEO_I2C_EXPBD_SPEED                         400000
#endif /* USE_STM32F4XX_NUCLEO or USE_STM32L1XX_NUCLEO */

/* Timing samples for L0 with SYSCLK 32MHz set in SystemClock_Config() */
#if (defined (TARGET_STM32L0))
#define NUCLEO_I2C_EXPBD_TIMING_100KHZ       0x10A13E56 /* Analog Filter ON, Rise Time 400ns, Fall Time 100ns */
#define NUCLEO_I2C_EXPBD_TIMING_400KHZ       0x00B1112E /* Analog Filter ON, Rise Time 250ns, Fall Time 100ns */
#endif /* TARGET_STM32L0 */

/* Timing samples for L4 with SYSCLK 80MHz set in SystemClock_Config() */
#if (defined (USE_STM32L4XX_NUCLEO))
#define NUCLEO_I2C_EXPBD_TIMING_400KHZ       0x10D1143A /* Analog Filter ON, Rise time 250ns, Fall Time 100ns */
#define NUCLEO_I2C_EXPBD_TIMING_1000KHZ      0x00D00E28 /* Analog Filter ON, Rise time 120ns, Fall time 25ns */
#endif /* USE_STM32L4XX_NUCLEO */

/* I2C peripheral configuration defines */
#define NUCLEO_I2C_EXPBD                            I2C1
#define NUCLEO_I2C_EXPBD_CLK_ENABLE()               __I2C1_CLK_ENABLE()
#define NUCLEO_I2C_EXPBD_SCL_SDA_GPIO_CLK_ENABLE()  __GPIOB_CLK_ENABLE()
#define NUCLEO_I2C_EXPBD_SCL_SDA_AF                 GPIO_AF4_I2C1
#define NUCLEO_I2C_EXPBD_SCL_SDA_GPIO_PORT          GPIOB
#define NUCLEO_I2C_EXPBD_SCL_PIN                    GPIO_PIN_8
#define NUCLEO_I2C_EXPBD_SDA_PIN                    GPIO_PIN_9

#define NUCLEO_I2C_EXPBD_FORCE_RESET()              __I2C1_FORCE_RESET()
#define NUCLEO_I2C_EXPBD_RELEASE_RESET()            __I2C1_RELEASE_RESET()

/* I2C interrupt requests */
#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define NUCLEO_I2C_EXPBD_EV_IRQn                    I2C1_EV_IRQn
#define NUCLEO_I2C_EXPBD_ER_IRQn                    I2C1_ER_IRQn
#endif

//#if (defined (TARGET_STM32L0))
#if (defined (TARGET_STM32L0))
#define NUCLEO_I2C_EXPBD_EV_IRQn                    I2C1_IRQn
#endif

/* Maximum Timeout values for flags waiting loops. These timeouts are not based
   on accurate values, they just guarantee that the application will not remain
   stuck if the I2C communication is corrupted.
   You may modify these timeout values depending on CPU frequency and application
   conditions (interrupts routines ...). */
#define NUCLEO_I2C_EXPBD_TIMEOUT_MAX    0x1000 /*<! The value of the maximal timeout for BUS waiting loops */

/* Definition for interrupt Pins */
#define LPS22H_INT1_O_GPIO_PORT           GPIOB
#define LPS22H_INT1_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define LPS22H_INT1_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define LPS22H_INT1_O_PIN                 GPIO_PIN_10

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define LPS22H_INT1_O_EXTI_IRQn           EXTI15_10_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define LPS22H_INT1_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

#define LSM6DSL_INT1_O_GPIO_PORT           GPIOB
#define LSM6DSL_INT1_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define LSM6DSL_INT1_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define LSM6DSL_INT1_O_PIN                 GPIO_PIN_5

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define LSM6DSL_INT1_O_EXTI_IRQn           EXTI9_5_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define LSM6DSL_INT1_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

#if 0
#define M_INT1_O_GPIO_PORT           GPIOC
#define M_INT1_O_GPIO_CLK_ENABLE()   __GPIOC_CLK_ENABLE()
#define M_INT1_O_GPIO_CLK_DISABLE()  __GPIOC_CLK_DISABLE()
#define M_INT1_O_PIN                 GPIO_PIN_0
#endif /* if 0 */
#define M_INT1_O_GPIO_PORT           GPIOB
#define M_INT1_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define M_INT1_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define M_INT1_O_PIN                 GPIO_PIN_7

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define M_INT1_O_EXTI_IRQn           EXTI0_IRQn
#endif

/*
#if (defined (TARGET_STM32L0))
#define M_INT1_O_EXTI_IRQn           EXTI0_1_IRQn
#endif
*/
#ifdef TARGET_DISCO_L072CZ_LRWAN1
#define M_INT1_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

#if 0
#define M_INT2_O_GPIO_PORT           GPIOC
#define M_INT2_O_GPIO_CLK_ENABLE()   __GPIOC_CLK_ENABLE()
#define M_INT2_O_GPIO_CLK_DISABLE()  __GPIOC_CLK_DISABLE()
#define M_INT2_O_PIN                 GPIO_PIN_1
#endif /* if 0 */
#define M_INT2_O_GPIO_PORT           GPIOB
#define M_INT2_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define M_INT2_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define M_INT2_O_PIN                 GPIO_PIN_6

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define M_INT2_O_EXTI_IRQn           EXTI1_IRQn
#endif

/*
#if (defined (TARGET_STM32L0))
#define M_INT2_O_EXTI_IRQn           EXTI0_1_IRQn
#endif
*/
#ifdef TARGET_DISCO_L072CZ_LRWAN1
#define M_INT2_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

#define LSM6DSL_INT2_O_GPIO_PORT           GPIOB
#define LSM6DSL_INT2_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define LSM6DSL_INT2_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define LSM6DSL_INT2_O_PIN                 GPIO_PIN_4

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define LSM6DSL_INT2_O_EXTI_IRQn           EXTI4_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define LSM6DSL_INT2_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

// ready for use
#define USER_INT_O_GPIO_PORT           GPIOA
#define USER_INT_O_GPIO_CLK_ENABLE()   __GPIOA_CLK_ENABLE()
#define USER_INT_O_GPIO_CLK_DISABLE()  __GPIOA_CLK_DISABLE()
#define USER_INT_O_PIN                 GPIO_PIN_10

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define USER_INT_O_EXTI_IRQn           EXTI15_10_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define USER_INT_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

// ready for use
#define LSM303AGR_DRDY_O_GPIO_PORT           GPIOA
#define LSM303AGR_DRDY_O_GPIO_CLK_ENABLE()   __GPIOA_CLK_ENABLE()
#define LSM303AGR_DRDY_O_GPIO_CLK_DISABLE()  __GPIOA_CLK_DISABLE()
#define LSM303AGR_DRDY_O_PIN                 GPIO_PIN_4

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define LSM303AGR_DRDY_O_EXTI_IRQn           EXTI4_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define LSM303AGR_DRDY_O_EXTI_IRQn           EXTI4_15_IRQn
#endif

// ready for use
#define LSM303AGR_INT_O_GPIO_PORT           GPIOB
#define LSM303AGR_INT_O_GPIO_CLK_ENABLE()   __GPIOB_CLK_ENABLE()
#define LSM303AGR_INT_O_GPIO_CLK_DISABLE()  __GPIOB_CLK_DISABLE()
#define LSM303AGR_INT_O_PIN                 GPIO_PIN_0

#if ((defined (USE_STM32F4XX_NUCLEO)) || (defined (USE_STM32L1XX_NUCLEO)) || (defined (USE_STM32L4XX_NUCLEO)))
#define LSM303AGR_INT_O_EXTI_IRQn           EXTI0_IRQn
#endif

#if (defined (TARGET_STM32L0))
#define LSM303AGR_INT_O_EXTI_IRQn           EXTI0_1_IRQn
#endif

/**
  * @}
  */

/** @addtogroup X_NUCLEO_IKS01A2_IO_Public_FunctionPrototypes Public function prototypes
 * @{
 */

DrvStatusTypeDef Sensor_IO_Init(void);
DrvStatusTypeDef LSM6DSL_Sensor_IO_ITConfig(void);
DrvStatusTypeDef LPS22HB_Sensor_IO_ITConfig(void);
DrvStatusTypeDef LSM303AGR_Sensor_IO_ITConfig(void);
DrvStatusTypeDef DIL24_Sensor_IO_ITConfig(void);

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif /* __X_NUCLEO_IKS01A2_H */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
